<?php
namespace ID3\Aliquando;

class Main extends \ID3\Bases\MainInstance{

    function __construct(){
        parent::__construct(__NAMESPACE__);

        $this->applyHooks();
        $this->applyCustomizations();

        $this->applyCodeFragments();
    }

    function applyHooks(){
        if(!is_admin()){
            $this->loadSettings();
            $this->applyLicensesHook();
            $this->applyCookieConsentHook();
            
        }

        if ( !is_super_admin() ) {
            remove_all_actions('admin_notices');
        }
        
    }

    

    function get_staging_site($domain){
        $args = array(
            'number'                 => 1,
            'update_site_meta_cache' => false,
        );
 
        $args['domain'] = str_replace('staging.','',$domain);
        
        $result = get_sites( $args );
        $site   = array_shift( $result );
     
        if ( $site ) {
            return $site;
        }
        return false;
    }



    function applyCustomizations(){
        $files = wp_get_upload_dir();
        $customize_folder = $files['basedir'] . '/ali_customize';
        $customize_folder_url = $files['baseurl'] . '/ali_customize';

        $this->initCustomizeFiles($customize_folder);
        
        // load customize functions file
        if(file_exists($customize_folder . '/functions.php')){
            include_once $customize_folder . '/functions.php';
        }
        

        if(!is_admin()){

            if(file_exists($customize_folder)){
                add_action('wp_footer', function() use ($customize_folder, $customize_folder_url){
                    if(file_exists($customize_folder . '/custom.min.css')){
                        echo('<link href="' . $customize_folder_url . '/custom.min.css" rel="stylesheet" />');
                    }

                    if(file_exists($customize_folder . '/custom.js')){
                        echo('<script type="text/javascript" src="' . $customize_folder_url . '/custom.js"></script>');
                    }
                });
            }
        }

        $this->applyNetworkMediaLibrary();
    }

    function applyCodeFragments(){
        if(!is_admin()){
            $this->loadSettings();
            $codeFragmentList = [];

            if(isset($this->settings->code_fragments)) $codeFragmentList = $this->settings->code_fragments;
            if(isset($this->networkSettings->code_fragments)){
                foreach($this->networkSettings->code_fragments as $fragment){
                    $codeFragmentList[] = $fragment;
                }
            }
            //\ID3\debug($codeFragmentList);

            foreach ($codeFragmentList as $fragment){
                $fragment = new CodeFragment($fragment);
                
                if($fragment->part == 'wp_action'){
                    add_action($fragment->action_name, function() use ($fragment){

                        $this->executeCodeFragment($fragment);
                        
                    });
                }
                else{
                    add_action('aliquando/' . $fragment->part, function() use ($fragment){
                        
                        $this->executeCodeFragment($fragment);
                        
                    });
                }
            }
        }
    }

    function applyCookieConsentHook(){
        $addHook = false;
        $consentDisplayType = '';
        $consentDisplayDelay = 0;

        if(isset($this->settings->cookie_consent)){
            if($this->settings->cookie_consent=='none') return;

            if(in_array($this->settings->cookie_consent,['message_bar','popup'])){
                $addHook = true;
                $consentDisplayType = $this->settings->cookie_consent;
                $consentDisplayDelay = $this->settings->cookie_consent_delay;
            }
        }

        if($addHook==false && isset($this->networkSettings->cookie_consent) && in_array($this->networkSettings->cookie_consent,['message_bar','popup'])){
            $addHook = true;
            $consentDisplayType = $this->networkSettings->cookie_consent;
            $consentDisplayDelay = $this->networkSettings->cookie_consent_delay;
        }

        if($addHook){

            add_action('aliquando/body:end', function() use ($consentDisplayType,$consentDisplayDelay){
                $privacyPolicyPageUrl = null;
                if(isset($this->settings->privacy_policies_page)){
                    if($this->settings->privacy_policies_page != null && ((int) $this->settings->privacy_policies_page > 0)){
                        $privacyPolicyPageUrl = \get_permalink( (int) $this->settings->privacy_policies_page);
                        $privacyPolicyPageUrl = apply_filters( 'wpml_permalink', $privacyPolicyPageUrl );

                    }
                }

                $defaultPath = PLUGIN_DIR . "views/php/cookie_consent_{$consentDisplayType}.php";
                $consentLayoutPath = apply_filters("id3-aliquando/layout/cookie_consent_{$consentDisplayType}", $defaultPath);
                if(file_exists($consentLayoutPath)){
                    include($consentLayoutPath);
                }
            });
        }
    }

    function applyNetworkMediaLibrary(){
        $this->loadSettings();

        if(!isset($this->settings->source_site) || \ID3\IsNullOrEmpty($this->settings->source_site)) return;
        
        // check if the source site is sharing it's media
        switch_to_blog( $this->settings->source_site );
        $sourceSettings = \ID3\Settings::get(KEY);
        restore_current_blog();

        if(false === $sourceSettings->share_media) return;

        include_once dirname(__FILE__) . '/class.network-media.php';
        new NetworkMediaLibrary($this->settings->source_site);
    }

    function applyLicensesHook(){
        
        $licenses =  [];
        $networkLicenses = [];
        if(isset($this->networkSettings->licenses) && is_array($this->networkSettings->licenses)){
            $networkLicenses = $this->networkSettings->licenses;
        }
        
        if(isset($this->settings->licenses) && is_array($this->settings->licenses)){
            foreach($this->settings->licenses as $lic){
                $networkLicense = array_values( array_filter($networkLicenses, function($uniqueObject) use ($lic) {
                    return $uniqueObject->key == $lic->key;
                }));
                
                if(isset($lic->value) && !\ID3\isNullOrEmpty($lic->value)){
                    $licenses[] = $lic;
                    continue;
                }

                if($networkLicense && count($networkLicense)>0){
                    $licenses[] = $networkLicense[0];
                }
            }
        }

        
        foreach($networkLicenses as $networkLicense){
            $siteLicense = array_filter($licenses, function($uniqueObject) use ($networkLicense) {
                return $uniqueObject->key == $networkLicense->key;
            });

            if($siteLicense) continue;

            if(!$networkLicense->activation_required){
                $licenses[] = $networkLicense;
            }
        }


        foreach($licenses as $lic){
            add_filter('licenses/' . $lic->key, function($value) use ($lic){
                return $lic->value;
            });
        }
    }

    function checkCodeFragmentConditions(CodeFragment $fragment){
        if(!$fragment->hasConditions()) return;

        return $fragment->conditionsMet();

    }

    function executeCodeFragment(CodeFragment $fragment){

        if($fragment->hasConditions()){
            if(!$fragment->conditionsMet()){
                
                return;
            }
        }
        

        if($fragment->code_language === 'php'){
            eval($fragment->code);
            return;
        }

        $codeWrap = ['',''];
        $content = do_shortcode($fragment->code);
        if($fragment->code_language === 'js'){
            $codeWrap = ['<script type="text/javascript">','</script>'];
        }
        else if($fragment->code_language === 'css'){
            $codeWrap = ['<style>','</style>'];
        }
        else if($fragment->code_language === 'html'){
            if(strpos($content,'<') !== 0){
                $codeWrap = ['<span>','</span>'];
            }
        }
        
        $result = implode($content,$codeWrap);
        echo($result);
    }

    function initCustomizeFiles($root=''){
        if($root == '') return;

        if(!file_exists($root)){
            try {
                \ID3\write_log('Creating customize folder: ' . $root);
                if(mkdir($root, 0777, true)){
                    // create scss file
                    $phpFile = fopen($root . "/functions.php", "w");
                    fwrite($phpFile,'<?php // silence is golden');
                    fclose($phpFile);

                    // create scss file
                    $scssFile = fopen($root . "/custom.scss", "w");
                    fwrite($scssFile,'// silence is golden');
                    fclose($scssFile);


                    // create js file
                    $jsFile = fopen($root . "/custom.js", "w");
                    fwrite($jsFile,'// silence is golden');
                    fclose($jsFile);
                }
            } catch (\Throwable $th) {
                \ID3\write_log('initCustomizeFiles: Failed to create customize directory - ' . $th->getMessage());
            }
        }
    }


}

new Main();


// Code Fragment
class CodeFragment {
    
    public $part;
    public $code;
    public $code_language;
    public $action_name;
    public $conditions = [];

    function __construct($data=null){
        //$this = $data;
        
        $blueprint = json_decode(json_encode($data), true);
        foreach ($blueprint as $key => $value) {
            $this->{$key} = $data->{$key};
        }
    }

    function hasConditions(){
        return count($this->conditions) > 0;
    }

    function conditionsMet(){
        foreach ($this->conditions as $cond){
            $condMethod = $cond->type . 'ConditionMet';
            if(method_exists($this,$condMethod)){
                if($this->{$condMethod}($cond->operator, $cond->value)){
                    return true;
                }
            }
        }

        return false;
    }

    function localeConditionMet($operator, $value){
        $locale = substr(get_locale(), 0, 2);
        return $this->_compareCondition($locale, $operator, $value);
    }

    function postIdConditionMet($operator, $value){
        $post_id = get_queried_object_id();
        return $this->_compareCondition($post_id, $operator, $value);
    }

    function slugConditionMet($operator, $value){
        $slug = get_queried_object()->post_name;
        
        return $this->_compareCondition($slug, $operator, $value);
    }

    function pathConditionMet($operator, $value){
        $path = trim($_SERVER['REQUEST_URI'],'/');
        $value = trim($value,'/');
        
        return $this->_compareCondition($path, $operator, $value);
    }

    private function _compareCondition($actual, $operator, $requested){
        $requested = explode(',', $requested);
        // unique value
        if($operator == 'equal') return in_array($actual,$requested);
        if($operator == 'not_equal') return !in_array($actual,$requested);    
    
        return false;
    }
}


// For Themes
class ThemeExtension {
    private $theme_base_path = '';

    public $working_directory = '/aliquando/';
    public $required_fields_view = 'requirements';
    public $optional_fields_view = 'optionals';

    public $settings = null;    // global settings from Aliquando + Theme settings
    public $network_settings = null;    // Network settings from Aliquando
    public $default_settings = [];

    function __construct($theme_path){
        $this->theme_base_path = $theme_path;
  
        if(!is_admin()){
            $this->loadSettings();
        }

        $this->register_hooks();
    }


    private function loadSettings(){
        if($this->settings != null) return;

        $this->settings = \ID3\Settings::get(constant(__NAMESPACE__ . '\KEY'));
        $this->network_settings = \ID3\Settings::getNetwork(constant(__NAMESPACE__ . '\KEY'));
    }

    private function register_hooks(){
        add_action('id3-aliquando/requirement/add', function() {
            $path = $this->theme_base_path . $this->working_directory . $this->required_fields_view . '.html';
            

            if(file_exists($path)){
                Admin::addRequirement($path, $this->default_settings);
            }
        });

        add_action('id3-aliquando/optional/add', function(){
            $path = $this->theme_base_path . $this->working_directory . $this->optional_fields_view . '.html';
            
            if(file_exists($path)){
                Admin::addOptional($path);
            }
        }); 

        add_action('id3-aliquando/theme/post-process', function($contextual_data){
            if(!method_exists($this, 'post_site_creation')) return;

            $this->post_site_creation($contextual_data);

        });
        
        add_action('id3-aliquando/register', function(){
            if(!method_exists($this, 'register')) return;
            $this->register();
        });

        add_action('wp_enqueue_scripts', function(){
            $path = $this->theme_base_path . $this->working_directory;
            $base_uri = str_replace([trim(\ABSPATH,'/'),'\\'], ['','/'], $path);
            
            if(strpos($base_uri,'//') == 0) $base_uri = str_replace('//','/', $base_uri);

            if(file_exists($path . 'styles.min.css')){
                wp_enqueue_style( 'script', "{$base_uri}styles.min.css");
            }
            
            if(file_exists($path . 'scripts.min.js')){
                wp_enqueue_script( 'script', "{$base_uri}scripts.min.js", null, 1.1, true);
            }

        });
        
    }
    

}
